# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from .schema_validation import (
    SchemaValidationResource,
    AsyncSchemaValidationResource,
    SchemaValidationResourceWithRawResponse,
    AsyncSchemaValidationResourceWithRawResponse,
    SchemaValidationResourceWithStreamingResponse,
    AsyncSchemaValidationResourceWithStreamingResponse,
)

__all__ = ["SettingsResource", "AsyncSettingsResource"]


class SettingsResource(SyncAPIResource):
    @cached_property
    def schema_validation(self) -> SchemaValidationResource:
        return SchemaValidationResource(self._client)

    @cached_property
    def with_raw_response(self) -> SettingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SettingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SettingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SettingsResourceWithStreamingResponse(self)


class AsyncSettingsResource(AsyncAPIResource):
    @cached_property
    def schema_validation(self) -> AsyncSchemaValidationResource:
        return AsyncSchemaValidationResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSettingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSettingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSettingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSettingsResourceWithStreamingResponse(self)


class SettingsResourceWithRawResponse:
    def __init__(self, settings: SettingsResource) -> None:
        self._settings = settings

    @cached_property
    def schema_validation(self) -> SchemaValidationResourceWithRawResponse:
        return SchemaValidationResourceWithRawResponse(self._settings.schema_validation)


class AsyncSettingsResourceWithRawResponse:
    def __init__(self, settings: AsyncSettingsResource) -> None:
        self._settings = settings

    @cached_property
    def schema_validation(self) -> AsyncSchemaValidationResourceWithRawResponse:
        return AsyncSchemaValidationResourceWithRawResponse(self._settings.schema_validation)


class SettingsResourceWithStreamingResponse:
    def __init__(self, settings: SettingsResource) -> None:
        self._settings = settings

    @cached_property
    def schema_validation(self) -> SchemaValidationResourceWithStreamingResponse:
        return SchemaValidationResourceWithStreamingResponse(self._settings.schema_validation)


class AsyncSettingsResourceWithStreamingResponse:
    def __init__(self, settings: AsyncSettingsResource) -> None:
        self._settings = settings

    @cached_property
    def schema_validation(self) -> AsyncSchemaValidationResourceWithStreamingResponse:
        return AsyncSchemaValidationResourceWithStreamingResponse(self._settings.schema_validation)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...types.zones import subscription_create_params, subscription_update_params
from ..._base_client import make_request_options
from ...types.shared_params.rate_plan import RatePlan
from ...types.zones.subscription_get_response import SubscriptionGetResponse
from ...types.zones.subscription_create_response import SubscriptionCreateResponse
from ...types.zones.subscription_update_response import SubscriptionUpdateResponse

__all__ = ["SubscriptionsResource", "AsyncSubscriptionsResource"]


class SubscriptionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SubscriptionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SubscriptionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SubscriptionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SubscriptionsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        zone_id: str,
        frequency: Literal["weekly", "monthly", "quarterly", "yearly"] | Omit = omit,
        rate_plan: RatePlan | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubscriptionCreateResponse:
        """
        Create a zone subscription, either plan or add-ons.

        Args:
          zone_id: Identifier

          frequency: How often the subscription is renewed automatically.

          rate_plan: The rate plan applied to the subscription.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._post(
            path_template("/zones/{zone_id}/subscription", zone_id=zone_id),
            body=maybe_transform(
                {
                    "frequency": frequency,
                    "rate_plan": rate_plan,
                },
                subscription_create_params.SubscriptionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SubscriptionCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[SubscriptionCreateResponse], ResultWrapper[SubscriptionCreateResponse]),
        )

    def update(
        self,
        *,
        zone_id: str,
        frequency: Literal["weekly", "monthly", "quarterly", "yearly"] | Omit = omit,
        rate_plan: RatePlan | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubscriptionUpdateResponse:
        """
        Updates zone subscriptions, either plan or add-ons.

        Args:
          zone_id: Identifier

          frequency: How often the subscription is renewed automatically.

          rate_plan: The rate plan applied to the subscription.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._put(
            path_template("/zones/{zone_id}/subscription", zone_id=zone_id),
            body=maybe_transform(
                {
                    "frequency": frequency,
                    "rate_plan": rate_plan,
                },
                subscription_update_params.SubscriptionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SubscriptionUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[SubscriptionUpdateResponse], ResultWrapper[SubscriptionUpdateResponse]),
        )

    def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubscriptionGetResponse:
        """
        Lists zone subscription details.

        Args:
          zone_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get(
            path_template("/zones/{zone_id}/subscription", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SubscriptionGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[SubscriptionGetResponse], ResultWrapper[SubscriptionGetResponse]),
        )


class AsyncSubscriptionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSubscriptionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSubscriptionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSubscriptionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSubscriptionsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        zone_id: str,
        frequency: Literal["weekly", "monthly", "quarterly", "yearly"] | Omit = omit,
        rate_plan: RatePlan | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubscriptionCreateResponse:
        """
        Create a zone subscription, either plan or add-ons.

        Args:
          zone_id: Identifier

          frequency: How often the subscription is renewed automatically.

          rate_plan: The rate plan applied to the subscription.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._post(
            path_template("/zones/{zone_id}/subscription", zone_id=zone_id),
            body=await async_maybe_transform(
                {
                    "frequency": frequency,
                    "rate_plan": rate_plan,
                },
                subscription_create_params.SubscriptionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SubscriptionCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[SubscriptionCreateResponse], ResultWrapper[SubscriptionCreateResponse]),
        )

    async def update(
        self,
        *,
        zone_id: str,
        frequency: Literal["weekly", "monthly", "quarterly", "yearly"] | Omit = omit,
        rate_plan: RatePlan | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubscriptionUpdateResponse:
        """
        Updates zone subscriptions, either plan or add-ons.

        Args:
          zone_id: Identifier

          frequency: How often the subscription is renewed automatically.

          rate_plan: The rate plan applied to the subscription.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._put(
            path_template("/zones/{zone_id}/subscription", zone_id=zone_id),
            body=await async_maybe_transform(
                {
                    "frequency": frequency,
                    "rate_plan": rate_plan,
                },
                subscription_update_params.SubscriptionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SubscriptionUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[SubscriptionUpdateResponse], ResultWrapper[SubscriptionUpdateResponse]),
        )

    async def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubscriptionGetResponse:
        """
        Lists zone subscription details.

        Args:
          zone_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._get(
            path_template("/zones/{zone_id}/subscription", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SubscriptionGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[SubscriptionGetResponse], ResultWrapper[SubscriptionGetResponse]),
        )


class SubscriptionsResourceWithRawResponse:
    def __init__(self, subscriptions: SubscriptionsResource) -> None:
        self._subscriptions = subscriptions

        self.create = to_raw_response_wrapper(
            subscriptions.create,
        )
        self.update = to_raw_response_wrapper(
            subscriptions.update,
        )
        self.get = to_raw_response_wrapper(
            subscriptions.get,
        )


class AsyncSubscriptionsResourceWithRawResponse:
    def __init__(self, subscriptions: AsyncSubscriptionsResource) -> None:
        self._subscriptions = subscriptions

        self.create = async_to_raw_response_wrapper(
            subscriptions.create,
        )
        self.update = async_to_raw_response_wrapper(
            subscriptions.update,
        )
        self.get = async_to_raw_response_wrapper(
            subscriptions.get,
        )


class SubscriptionsResourceWithStreamingResponse:
    def __init__(self, subscriptions: SubscriptionsResource) -> None:
        self._subscriptions = subscriptions

        self.create = to_streamed_response_wrapper(
            subscriptions.create,
        )
        self.update = to_streamed_response_wrapper(
            subscriptions.update,
        )
        self.get = to_streamed_response_wrapper(
            subscriptions.get,
        )


class AsyncSubscriptionsResourceWithStreamingResponse:
    def __init__(self, subscriptions: AsyncSubscriptionsResource) -> None:
        self._subscriptions = subscriptions

        self.create = async_to_streamed_response_wrapper(
            subscriptions.create,
        )
        self.update = async_to_streamed_response_wrapper(
            subscriptions.update,
        )
        self.get = async_to_streamed_response_wrapper(
            subscriptions.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from ..._utils import PropertyInfo

__all__ = [
    "PresetCreateParams",
    "Config",
    "ConfigMaxVideoStreams",
    "ConfigMedia",
    "ConfigMediaScreenshare",
    "ConfigMediaVideo",
    "ConfigMediaAudio",
    "UI",
    "UIDesignTokens",
    "UIDesignTokensColors",
    "UIDesignTokensColorsBackground",
    "UIDesignTokensColorsBrand",
    "Permissions",
    "PermissionsChat",
    "PermissionsChatPrivate",
    "PermissionsChatPublic",
    "PermissionsConnectedMeetings",
    "PermissionsMedia",
    "PermissionsMediaAudio",
    "PermissionsMediaScreenshare",
    "PermissionsMediaVideo",
    "PermissionsPlugins",
    "PermissionsPluginsConfig",
    "PermissionsPluginsConfigUnionMember1",
    "PermissionsPolls",
]


class PresetCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """The account identifier tag."""

    config: Required[Config]

    name: Required[str]
    """Name of the preset"""

    ui: Required[UI]

    permissions: Permissions


class ConfigMaxVideoStreams(TypedDict, total=False):
    """Maximum number of streams that are visible on a device"""

    desktop: Required[int]
    """Maximum number of video streams visible on desktop devices"""

    mobile: Required[int]
    """Maximum number of streams visible on mobile devices"""


class ConfigMediaScreenshare(TypedDict, total=False):
    """Configuration options for participant screen shares"""

    frame_rate: Required[int]
    """Frame rate of screen share"""

    quality: Required[Literal["hd", "vga", "qvga"]]
    """Quality of screen share"""


class ConfigMediaVideo(TypedDict, total=False):
    """Configuration options for participant videos"""

    frame_rate: Required[int]
    """Frame rate of participants' video"""

    quality: Required[Literal["hd", "vga", "qvga"]]
    """Video quality of participants"""


class ConfigMediaAudio(TypedDict, total=False):
    """Control options for Audio quality."""

    enable_high_bitrate: bool
    """Enable High Quality Audio for your meetings"""

    enable_stereo: bool
    """Enable Stereo for your meetings"""


class ConfigMedia(TypedDict, total=False):
    """Media configuration options. eg: Video quality"""

    screenshare: Required[ConfigMediaScreenshare]
    """Configuration options for participant screen shares"""

    video: Required[ConfigMediaVideo]
    """Configuration options for participant videos"""

    audio: ConfigMediaAudio
    """Control options for Audio quality."""


class Config(TypedDict, total=False):
    max_screenshare_count: Required[int]
    """Maximum number of screen shares that can be active at a given time"""

    max_video_streams: Required[ConfigMaxVideoStreams]
    """Maximum number of streams that are visible on a device"""

    media: Required[ConfigMedia]
    """Media configuration options. eg: Video quality"""

    view_type: Required[Literal["GROUP_CALL", "WEBINAR", "AUDIO_ROOM"]]
    """Type of the meeting"""


class UIDesignTokensColorsBackground(TypedDict, total=False):
    _1000: Required[Annotated[str, PropertyInfo(alias="1000")]]

    _600: Required[Annotated[str, PropertyInfo(alias="600")]]

    _700: Required[Annotated[str, PropertyInfo(alias="700")]]

    _800: Required[Annotated[str, PropertyInfo(alias="800")]]

    _900: Required[Annotated[str, PropertyInfo(alias="900")]]


class UIDesignTokensColorsBrand(TypedDict, total=False):
    _300: Required[Annotated[str, PropertyInfo(alias="300")]]

    _400: Required[Annotated[str, PropertyInfo(alias="400")]]

    _500: Required[Annotated[str, PropertyInfo(alias="500")]]

    _600: Required[Annotated[str, PropertyInfo(alias="600")]]

    _700: Required[Annotated[str, PropertyInfo(alias="700")]]


class UIDesignTokensColors(TypedDict, total=False):
    background: Required[UIDesignTokensColorsBackground]

    brand: Required[UIDesignTokensColorsBrand]

    danger: Required[str]

    success: Required[str]

    text: Required[str]

    text_on_brand: Required[str]

    video_bg: Required[str]

    warning: Required[str]


class UIDesignTokens(TypedDict, total=False):
    border_radius: Required[Literal["rounded"]]

    border_width: Required[Literal["thin"]]

    colors: Required[UIDesignTokensColors]

    logo: Required[str]

    spacing_base: Required[float]

    theme: Required[Literal["dark"]]


class UI(TypedDict, total=False):
    design_tokens: Required[UIDesignTokens]

    config_diff: object


class PermissionsChatPrivate(TypedDict, total=False):
    can_receive: Required[bool]

    can_send: Required[bool]

    files: Required[bool]

    text: Required[bool]


class PermissionsChatPublic(TypedDict, total=False):
    can_send: Required[bool]
    """Can send messages in general"""

    files: Required[bool]
    """Can send file messages"""

    text: Required[bool]
    """Can send text messages"""


class PermissionsChat(TypedDict, total=False):
    """Chat permissions"""

    private: Required[PermissionsChatPrivate]

    public: Required[PermissionsChatPublic]


class PermissionsConnectedMeetings(TypedDict, total=False):
    can_alter_connected_meetings: Required[bool]

    can_switch_connected_meetings: Required[bool]

    can_switch_to_parent_meeting: Required[bool]


class PermissionsMediaAudio(TypedDict, total=False):
    """Audio permissions"""

    can_produce: Required[Literal["ALLOWED", "NOT_ALLOWED", "CAN_REQUEST"]]
    """Can produce audio"""


class PermissionsMediaScreenshare(TypedDict, total=False):
    """Screenshare permissions"""

    can_produce: Required[Literal["ALLOWED", "NOT_ALLOWED", "CAN_REQUEST"]]
    """Can produce screen share video"""


class PermissionsMediaVideo(TypedDict, total=False):
    """Video permissions"""

    can_produce: Required[Literal["ALLOWED", "NOT_ALLOWED", "CAN_REQUEST"]]
    """Can produce video"""


class PermissionsMedia(TypedDict, total=False):
    """Media permissions"""

    audio: Required[PermissionsMediaAudio]
    """Audio permissions"""

    screenshare: Required[PermissionsMediaScreenshare]
    """Screenshare permissions"""

    video: Required[PermissionsMediaVideo]
    """Video permissions"""


class PermissionsPluginsConfigUnionMember1(TypedDict, total=False):
    access_control: Required[Literal["FULL_ACCESS", "VIEW_ONLY"]]

    handles_view_only: Required[bool]


PermissionsPluginsConfig: TypeAlias = Union[str, PermissionsPluginsConfigUnionMember1]


class PermissionsPlugins(TypedDict, total=False):
    """Plugin permissions"""

    can_close: Required[bool]
    """Can close plugins that are already open"""

    can_edit_config: Required[bool]
    """Can edit plugin config"""

    can_start: Required[bool]
    """Can start plugins"""

    config: Required[PermissionsPluginsConfig]


class PermissionsPolls(TypedDict, total=False):
    """Poll permissions"""

    can_create: Required[bool]
    """Can create polls"""

    can_view: Required[bool]
    """Can view polls"""

    can_vote: Required[bool]
    """Can vote on polls"""


class Permissions(TypedDict, total=False):
    accept_waiting_requests: Required[bool]
    """Whether this participant can accept waiting requests"""

    can_accept_production_requests: Required[bool]

    can_change_participant_permissions: Required[bool]

    can_edit_display_name: Required[bool]

    can_livestream: Required[bool]

    can_record: Required[bool]

    can_spotlight: Required[bool]

    chat: Required[PermissionsChat]
    """Chat permissions"""

    connected_meetings: Required[PermissionsConnectedMeetings]

    disable_participant_audio: Required[bool]

    disable_participant_screensharing: Required[bool]

    disable_participant_video: Required[bool]

    hidden_participant: Required[bool]
    """Whether this participant is visible to others or not"""

    kick_participant: Required[bool]

    media: Required[PermissionsMedia]
    """Media permissions"""

    pin_participant: Required[bool]

    plugins: Required[PermissionsPlugins]
    """Plugin permissions"""

    polls: Required[PermissionsPolls]
    """Poll permissions"""

    recorder_type: Required[Literal["RECORDER", "LIVESTREAMER", "NONE"]]
    """Type of the recording peer"""

    show_participant_list: Required[bool]

    waiting_room_type: Required[Literal["SKIP", "ON_PRIVILEGED_USER_ENTRY", "SKIP_ON_ACCEPT"]]
    """Waiting room type"""

    is_recorder: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ...._models import BaseModel

__all__ = ["SAMLCertificateGetResponse", "CurrentCertificate"]


class CurrentCertificate(BaseModel):
    """The current active certificate"""

    is_current: bool
    """Indicates whether the certificate can be used for IdP configuration."""

    not_after: datetime
    """Certificate expiration date"""

    public_certificate: str
    """The public certificate in PEM format"""

    uid: str
    """Unique identifier for the certificate"""


class SAMLCertificateGetResponse(BaseModel):
    created_at: datetime
    """When the certificate set was created"""

    uid: str
    """Unique identifier for the certificate set"""

    updated_at: datetime
    """When the certificate set was last updated"""

    current_certificate: Optional[CurrentCertificate] = None
    """The current active certificate"""

    previous_certificate: Optional[object] = None
    """The previous certificate (maintained during rotation period).

    May be null when no rotation has occurred. Mirrors the structure of
    `saml_certificate`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, Dict, Optional, cast
from typing_extensions import Literal, overload

import httpx

from ..._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ..._utils import path_template, required_args, maybe_transform, strip_not_given, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ..._base_client import make_request_options
from ...types.resource_tagging import zone_tag_get_params, zone_tag_update_params
from ...types.resource_tagging.zone_tag_get_response import ZoneTagGetResponse
from ...types.resource_tagging.zone_tag_update_response import ZoneTagUpdateResponse

__all__ = ["ZoneTagsResource", "AsyncZoneTagsResource"]


class ZoneTagsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ZoneTagsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ZoneTagsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ZoneTagsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ZoneTagsResourceWithStreamingResponse(self)

    @overload
    def update(
        self,
        *,
        zone_id: str,
        resource_id: str,
        resource_type: Literal[
            "api_gateway_operation",
            "custom_certificate",
            "custom_hostname",
            "dns_record",
            "managed_client_certificate",
            "zone",
        ],
        tags: Dict[str, str] | Omit = omit,
        if_match: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ZoneTagUpdateResponse]:
        """Creates or updates tags for a specific zone-level resource.

        Replaces all
        existing tags for the resource.

        Args:
          zone_id: Zone ID is required only for zone-level resources

          resource_id: Identifies the unique resource.

          resource_type: Enum for base zone-level resource types (those with no extra required fields).

          tags: Contains key-value pairs of tags.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def update(
        self,
        *,
        zone_id: str,
        access_application_id: str,
        resource_id: str,
        resource_type: Literal[
            "api_gateway_operation",
            "custom_certificate",
            "custom_hostname",
            "dns_record",
            "managed_client_certificate",
            "zone",
            "access_application_policy",
        ],
        tags: Dict[str, str] | Omit = omit,
        if_match: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ZoneTagUpdateResponse]:
        """Creates or updates tags for a specific zone-level resource.

        Replaces all
        existing tags for the resource.

        Args:
          zone_id: Zone ID is required only for zone-level resources

          access_application_id: Access application ID is required only for access_application_policy resources

          resource_id: Identifies the unique resource.

          resource_type: Enum for base zone-level resource types (those with no extra required fields).

          tags: Contains key-value pairs of tags.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(
        ["zone_id", "resource_id", "resource_type"],
        ["zone_id", "access_application_id", "resource_id", "resource_type"],
    )
    def update(
        self,
        *,
        zone_id: str,
        resource_id: str,
        resource_type: Literal[
            "api_gateway_operation",
            "custom_certificate",
            "custom_hostname",
            "dns_record",
            "managed_client_certificate",
            "zone",
        ]
        | Literal[
            "api_gateway_operation",
            "custom_certificate",
            "custom_hostname",
            "dns_record",
            "managed_client_certificate",
            "zone",
            "access_application_policy",
        ],
        tags: Dict[str, str] | Omit = omit,
        if_match: str | Omit = omit,
        access_application_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ZoneTagUpdateResponse]:
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        extra_headers = {**strip_not_given({"If-Match": if_match}), **(extra_headers or {})}
        return cast(
            Optional[ZoneTagUpdateResponse],
            self._put(
                path_template("/zones/{zone_id}/tags", zone_id=zone_id),
                body=maybe_transform(
                    {
                        "resource_id": resource_id,
                        "resource_type": resource_type,
                        "tags": tags,
                        "access_application_id": access_application_id,
                    },
                    zone_tag_update_params.ZoneTagUpdateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[ZoneTagUpdateResponse]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[ZoneTagUpdateResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def delete(
        self,
        *,
        zone_id: str,
        if_match: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Removes all tags from a specific zone-level resource.

        Args:
          zone_id: Zone ID is required only for zone-level resources

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        extra_headers = {**strip_not_given({"If-Match": if_match}), **(extra_headers or {})}
        return self._delete(
            path_template("/zones/{zone_id}/tags", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def get(
        self,
        *,
        zone_id: str,
        resource_id: str,
        resource_type: Literal[
            "access_application_policy",
            "api_gateway_operation",
            "custom_certificate",
            "custom_hostname",
            "dns_record",
            "managed_client_certificate",
            "zone",
        ],
        access_application_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ZoneTagGetResponse]:
        """
        Retrieves tags for a specific zone-level resource.

        Args:
          zone_id: Zone ID is required only for zone-level resources

          resource_id: The ID of the resource to retrieve tags for.

          resource_type: The type of the resource.

          access_application_id: Access application ID identifier. Required for access_application_policy
              resources.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return cast(
            Optional[ZoneTagGetResponse],
            self._get(
                path_template("/zones/{zone_id}/tags", zone_id=zone_id),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    query=maybe_transform(
                        {
                            "resource_id": resource_id,
                            "resource_type": resource_type,
                            "access_application_id": access_application_id,
                        },
                        zone_tag_get_params.ZoneTagGetParams,
                    ),
                    post_parser=ResultWrapper[Optional[ZoneTagGetResponse]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[ZoneTagGetResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class AsyncZoneTagsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncZoneTagsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncZoneTagsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncZoneTagsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncZoneTagsResourceWithStreamingResponse(self)

    @overload
    async def update(
        self,
        *,
        zone_id: str,
        resource_id: str,
        resource_type: Literal[
            "api_gateway_operation",
            "custom_certificate",
            "custom_hostname",
            "dns_record",
            "managed_client_certificate",
            "zone",
        ],
        tags: Dict[str, str] | Omit = omit,
        if_match: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ZoneTagUpdateResponse]:
        """Creates or updates tags for a specific zone-level resource.

        Replaces all
        existing tags for the resource.

        Args:
          zone_id: Zone ID is required only for zone-level resources

          resource_id: Identifies the unique resource.

          resource_type: Enum for base zone-level resource types (those with no extra required fields).

          tags: Contains key-value pairs of tags.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def update(
        self,
        *,
        zone_id: str,
        access_application_id: str,
        resource_id: str,
        resource_type: Literal[
            "api_gateway_operation",
            "custom_certificate",
            "custom_hostname",
            "dns_record",
            "managed_client_certificate",
            "zone",
            "access_application_policy",
        ],
        tags: Dict[str, str] | Omit = omit,
        if_match: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ZoneTagUpdateResponse]:
        """Creates or updates tags for a specific zone-level resource.

        Replaces all
        existing tags for the resource.

        Args:
          zone_id: Zone ID is required only for zone-level resources

          access_application_id: Access application ID is required only for access_application_policy resources

          resource_id: Identifies the unique resource.

          resource_type: Enum for base zone-level resource types (those with no extra required fields).

          tags: Contains key-value pairs of tags.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(
        ["zone_id", "resource_id", "resource_type"],
        ["zone_id", "access_application_id", "resource_id", "resource_type"],
    )
    async def update(
        self,
        *,
        zone_id: str,
        resource_id: str,
        resource_type: Literal[
            "api_gateway_operation",
            "custom_certificate",
            "custom_hostname",
            "dns_record",
            "managed_client_certificate",
            "zone",
        ]
        | Literal[
            "api_gateway_operation",
            "custom_certificate",
            "custom_hostname",
            "dns_record",
            "managed_client_certificate",
            "zone",
            "access_application_policy",
        ],
        tags: Dict[str, str] | Omit = omit,
        if_match: str | Omit = omit,
        access_application_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ZoneTagUpdateResponse]:
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        extra_headers = {**strip_not_given({"If-Match": if_match}), **(extra_headers or {})}
        return cast(
            Optional[ZoneTagUpdateResponse],
            await self._put(
                path_template("/zones/{zone_id}/tags", zone_id=zone_id),
                body=await async_maybe_transform(
                    {
                        "resource_id": resource_id,
                        "resource_type": resource_type,
                        "tags": tags,
                        "access_application_id": access_application_id,
                    },
                    zone_tag_update_params.ZoneTagUpdateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[ZoneTagUpdateResponse]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[ZoneTagUpdateResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def delete(
        self,
        *,
        zone_id: str,
        if_match: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Removes all tags from a specific zone-level resource.

        Args:
          zone_id: Zone ID is required only for zone-level resources

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        extra_headers = {**strip_not_given({"If-Match": if_match}), **(extra_headers or {})}
        return await self._delete(
            path_template("/zones/{zone_id}/tags", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def get(
        self,
        *,
        zone_id: str,
        resource_id: str,
        resource_type: Literal[
            "access_application_policy",
            "api_gateway_operation",
            "custom_certificate",
            "custom_hostname",
            "dns_record",
            "managed_client_certificate",
            "zone",
        ],
        access_application_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ZoneTagGetResponse]:
        """
        Retrieves tags for a specific zone-level resource.

        Args:
          zone_id: Zone ID is required only for zone-level resources

          resource_id: The ID of the resource to retrieve tags for.

          resource_type: The type of the resource.

          access_application_id: Access application ID identifier. Required for access_application_policy
              resources.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return cast(
            Optional[ZoneTagGetResponse],
            await self._get(
                path_template("/zones/{zone_id}/tags", zone_id=zone_id),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    query=await async_maybe_transform(
                        {
                            "resource_id": resource_id,
                            "resource_type": resource_type,
                            "access_application_id": access_application_id,
                        },
                        zone_tag_get_params.ZoneTagGetParams,
                    ),
                    post_parser=ResultWrapper[Optional[ZoneTagGetResponse]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[ZoneTagGetResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class ZoneTagsResourceWithRawResponse:
    def __init__(self, zone_tags: ZoneTagsResource) -> None:
        self._zone_tags = zone_tags

        self.update = to_raw_response_wrapper(
            zone_tags.update,
        )
        self.delete = to_raw_response_wrapper(
            zone_tags.delete,
        )
        self.get = to_raw_response_wrapper(
            zone_tags.get,
        )


class AsyncZoneTagsResourceWithRawResponse:
    def __init__(self, zone_tags: AsyncZoneTagsResource) -> None:
        self._zone_tags = zone_tags

        self.update = async_to_raw_response_wrapper(
            zone_tags.update,
        )
        self.delete = async_to_raw_response_wrapper(
            zone_tags.delete,
        )
        self.get = async_to_raw_response_wrapper(
            zone_tags.get,
        )


class ZoneTagsResourceWithStreamingResponse:
    def __init__(self, zone_tags: ZoneTagsResource) -> None:
        self._zone_tags = zone_tags

        self.update = to_streamed_response_wrapper(
            zone_tags.update,
        )
        self.delete = to_streamed_response_wrapper(
            zone_tags.delete,
        )
        self.get = to_streamed_response_wrapper(
            zone_tags.get,
        )


class AsyncZoneTagsResourceWithStreamingResponse:
    def __init__(self, zone_tags: AsyncZoneTagsResource) -> None:
        self._zone_tags = zone_tags

        self.update = async_to_streamed_response_wrapper(
            zone_tags.update,
        )
        self.delete = async_to_streamed_response_wrapper(
            zone_tags.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            zone_tags.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from .hostname_association import HostnameAssociation

__all__ = ["HostnameAssociationUpdateResponse"]


class HostnameAssociationUpdateResponse(BaseModel):
    hostnames: Optional[List[HostnameAssociation]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = ["OrganizationListParams", "Containing", "Name", "Parent"]


class OrganizationListParams(TypedDict, total=False):
    id: SequenceNotStr[str]
    """Only return organizations with the specified IDs (ex.

    id=foo&id=bar). Send multiple elements by repeating the query value.
    """

    containing: Containing

    name: Name

    page_size: int
    """The amount of items to return. Defaults to 10."""

    page_token: str
    """
    An opaque token returned from the last list response that when provided will
    retrieve the next page.

    Parameters used to filter the retrieved list must remain in subsequent requests
    with a page token.
    """

    parent: Parent


class Containing(TypedDict, total=False):
    account: str
    """
    Filter the list of organizations to the ones that contain this particular
    account.
    """

    organization: str
    """
    Filter the list of organizations to the ones that contain this particular
    organization.
    """

    user: str
    """Filter the list of organizations to the ones that contain this particular user.

    IMPORTANT: Just because an organization "contains" a user is not a
    representation of any authorization or privilege to manage any resources
    therein. An organization "containing" a user simply means the user is managed by
    that organization.
    """


class Name(TypedDict, total=False):
    contains: str
    """
    (case-insensitive) Filter the list of organizations to where the name contains a
    particular string.
    """

    ends_with: Annotated[str, PropertyInfo(alias="endsWith")]
    """
    (case-insensitive) Filter the list of organizations to where the name ends with
    a particular string.
    """

    starts_with: Annotated[str, PropertyInfo(alias="startsWith")]
    """
    (case-insensitive) Filter the list of organizations to where the name starts
    with a particular string.
    """


class Parent(TypedDict, total=False):
    id: Union[str, Literal["null"]]
    """
    Filter the list of organizations to the ones that are a sub-organization of the
    specified organization.

    "null" is a valid value to provide for this parameter. It means "where an
    organization has no parent (i.e. it is a 'root' organization)."
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["ObjectListResponse", "HTTPMetadata"]


class HTTPMetadata(BaseModel):
    """HTTP metadata associated with an R2 object."""

    cache_control: Optional[str] = FieldInfo(alias="cacheControl", default=None)
    """Specifies caching behavior for the object."""

    cache_expiry: Optional[datetime] = FieldInfo(alias="cacheExpiry", default=None)
    """The date and time at which the object's cache entry expires."""

    content_disposition: Optional[str] = FieldInfo(alias="contentDisposition", default=None)
    """Specifies presentational information for the object."""

    content_encoding: Optional[str] = FieldInfo(alias="contentEncoding", default=None)
    """Specifies the content encoding applied to the object."""

    content_language: Optional[str] = FieldInfo(alias="contentLanguage", default=None)
    """The language of the object content."""

    content_type: Optional[str] = FieldInfo(alias="contentType", default=None)
    """The MIME type of the object."""


class ObjectListResponse(BaseModel):
    """Metadata for an R2 object."""

    custom_metadata: Optional[Dict[str, str]] = None
    """Custom metadata key-value pairs associated with the object."""

    etag: Optional[str] = None
    """The entity tag for the object.

    In JSON list/get responses this is the raw hex digest (without surrounding
    quotes). The HTTP `ETag` response header on Get Object follows RFC 7232 and IS
    wrapped in surrounding double-quotes.
    """

    http_metadata: Optional[HTTPMetadata] = None
    """HTTP metadata associated with an R2 object."""

    key: Optional[str] = None
    """The object key (name)."""

    last_modified: Optional[datetime] = None
    """The date and time the object was last modified."""

    size: Optional[int] = None
    """The size of the object in bytes."""

    ssec: Optional[bool] = None
    """Whether the object is encrypted with a customer-supplied encryption key."""

    storage_class: Optional[Literal["Standard", "InfrequentAccess"]] = None
    """Storage class for newly uploaded objects, unless specified otherwise."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from ..custom_certificates.status import Status

__all__ = ["ClientCertificate", "CertificateAuthority"]


class CertificateAuthority(BaseModel):
    """Certificate Authority used to issue the Client Certificate"""

    id: Optional[str] = None

    name: Optional[str] = None


class ClientCertificate(BaseModel):
    id: Optional[str] = None
    """Identifier."""

    certificate: Optional[str] = None
    """The Client Certificate PEM"""

    certificate_authority: Optional[CertificateAuthority] = None
    """Certificate Authority used to issue the Client Certificate"""

    common_name: Optional[str] = None
    """Common Name of the Client Certificate"""

    country: Optional[str] = None
    """Country, provided by the CSR"""

    csr: Optional[str] = None
    """The Certificate Signing Request (CSR). Must be newline-encoded."""

    expires_on: Optional[str] = None
    """Date that the Client Certificate expires"""

    fingerprint_sha256: Optional[str] = None
    """Unique identifier of the Client Certificate"""

    issued_on: Optional[str] = None
    """Date that the Client Certificate was issued by the Certificate Authority"""

    location: Optional[str] = None
    """Location, provided by the CSR"""

    organization: Optional[str] = None
    """Organization, provided by the CSR"""

    organizational_unit: Optional[str] = None
    """Organizational Unit, provided by the CSR"""

    serial_number: Optional[str] = None
    """The serial number on the created Client Certificate."""

    signature: Optional[str] = None
    """The type of hash used for the Client Certificate.."""

    ski: Optional[str] = None
    """Subject Key Identifier"""

    state: Optional[str] = None
    """State, provided by the CSR"""

    status: Optional[Status] = None
    """
    Client Certificates may be active or revoked, and the pending_reactivation or
    pending_revocation represent in-progress asynchronous transitions
    """

    validity_days: Optional[int] = None
    """
    The number of days the Client Certificate will be valid after the issued_on date
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

__all__ = ["SeatEditParams", "Body"]


class SeatEditParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    body: Required[Iterable[Body]]


class Body(TypedDict, total=False):
    access_seat: Required[bool]
    """True if the seat is part of Access."""

    gateway_seat: Required[bool]
    """True if the seat is part of Gateway."""

    seat_uid: Required[str]
    """The unique API identifier for the Zero Trust seat."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["IndicatorFeedUpdateResponse"]


class IndicatorFeedUpdateResponse(BaseModel):
    id: Optional[int] = None
    """The unique identifier for the indicator feed"""

    created_on: Optional[datetime] = None
    """The date and time when the data entry was created"""

    description: Optional[str] = None
    """The description of the example test"""

    is_attributable: Optional[bool] = None
    """Whether the indicator feed can be attributed to a provider"""

    is_downloadable: Optional[bool] = None
    """Whether the indicator feed can be downloaded"""

    is_public: Optional[bool] = None
    """Whether the indicator feed is exposed to customers"""

    modified_on: Optional[datetime] = None
    """The date and time when the data entry was last modified"""

    name: Optional[str] = None
    """The name of the indicator feed"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, Type, Iterable, Optional, cast

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from ....._base_client import make_request_options
from .....types.zero_trust.dlp.profile import Profile
from .....types.zero_trust.dlp.profiles import custom_create_params, custom_update_params
from .....types.zero_trust.dlp.context_awareness_param import ContextAwarenessParam

__all__ = ["CustomResource", "AsyncCustomResource"]


class CustomResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CustomResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return CustomResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CustomResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return CustomResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        name: str,
        ai_context_enabled: bool | Omit = omit,
        allowed_match_count: int | Omit = omit,
        confidence_threshold: Optional[str] | Omit = omit,
        context_awareness: ContextAwarenessParam | Omit = omit,
        data_classes: SequenceNotStr[str] | Omit = omit,
        data_tags: SequenceNotStr[str] | Omit = omit,
        description: Optional[str] | Omit = omit,
        entries: Iterable[custom_create_params.Entry] | Omit = omit,
        ocr_enabled: bool | Omit = omit,
        sensitivity_levels: Iterable[custom_create_params.SensitivityLevel] | Omit = omit,
        shared_entries: Iterable[custom_create_params.SharedEntry] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Profile]:
        """
        Creates a DLP custom profile.

        Args:
          allowed_match_count: Related DLP policies will trigger when the match count exceeds the number set.

          context_awareness: Scan the context of predefined entries to only return matches surrounded by
              keywords.

          data_classes: Data class IDs to associate with the profile.

          data_tags: Data tag IDs to associate with the profile.

          description: The description of the profile.

          sensitivity_levels: Sensitivity levels to associate with the profile.

          shared_entries: Entries from other profiles (e.g. pre-defined Cloudflare profiles, or your
              Microsoft Information Protection profiles).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return cast(
            Optional[Profile],
            self._post(
                path_template("/accounts/{account_id}/dlp/profiles/custom", account_id=account_id),
                body=maybe_transform(
                    {
                        "name": name,
                        "ai_context_enabled": ai_context_enabled,
                        "allowed_match_count": allowed_match_count,
                        "confidence_threshold": confidence_threshold,
                        "context_awareness": context_awareness,
                        "data_classes": data_classes,
                        "data_tags": data_tags,
                        "description": description,
                        "entries": entries,
                        "ocr_enabled": ocr_enabled,
                        "sensitivity_levels": sensitivity_levels,
                        "shared_entries": shared_entries,
                    },
                    custom_create_params.CustomCreateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[Profile]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[Profile]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def update(
        self,
        profile_id: str,
        *,
        account_id: str,
        name: str,
        ai_context_enabled: bool | Omit = omit,
        allowed_match_count: Optional[int] | Omit = omit,
        confidence_threshold: Optional[str] | Omit = omit,
        context_awareness: ContextAwarenessParam | Omit = omit,
        data_classes: Optional[SequenceNotStr[str]] | Omit = omit,
        data_tags: Optional[SequenceNotStr[str]] | Omit = omit,
        description: Optional[str] | Omit = omit,
        entries: Optional[Iterable[custom_update_params.Entry]] | Omit = omit,
        ocr_enabled: bool | Omit = omit,
        sensitivity_levels: Optional[Iterable[custom_update_params.SensitivityLevel]] | Omit = omit,
        shared_entries: Iterable[custom_update_params.SharedEntry] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Profile]:
        """
        Updates a DLP custom profile.

        Args:
          context_awareness: Scan the context of predefined entries to only return matches surrounded by
              keywords.

          data_classes: Data class IDs to associate with the profile. If omitted, existing associations
              are unchanged.

          data_tags: Data tag IDs to associate with the profile. If omitted, existing associations
              are unchanged.

          description: The description of the profile.

          entries: Custom entries from this profile. If this field is omitted, entries owned by
              this profile will not be changed.

          sensitivity_levels: Sensitivity levels to associate with the profile. If omitted, existing
              associations are unchanged.

          shared_entries: Other entries, e.g. predefined or integration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not profile_id:
            raise ValueError(f"Expected a non-empty value for `profile_id` but received {profile_id!r}")
        return cast(
            Optional[Profile],
            self._put(
                path_template(
                    "/accounts/{account_id}/dlp/profiles/custom/{profile_id}",
                    account_id=account_id,
                    profile_id=profile_id,
                ),
                body=maybe_transform(
                    {
                        "name": name,
                        "ai_context_enabled": ai_context_enabled,
                        "allowed_match_count": allowed_match_count,
                        "confidence_threshold": confidence_threshold,
                        "context_awareness": context_awareness,
                        "data_classes": data_classes,
                        "data_tags": data_tags,
                        "description": description,
                        "entries": entries,
                        "ocr_enabled": ocr_enabled,
                        "sensitivity_levels": sensitivity_levels,
                        "shared_entries": shared_entries,
                    },
                    custom_update_params.CustomUpdateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[Profile]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[Profile]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def delete(
        self,
        profile_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Deletes a DLP custom profile.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not profile_id:
            raise ValueError(f"Expected a non-empty value for `profile_id` but received {profile_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/dlp/profiles/custom/{profile_id}", account_id=account_id, profile_id=profile_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    def get(
        self,
        profile_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Profile]:
        """
        Fetches a custom DLP profile by id.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not profile_id:
            raise ValueError(f"Expected a non-empty value for `profile_id` but received {profile_id!r}")
        return cast(
            Optional[Profile],
            self._get(
                path_template(
                    "/accounts/{account_id}/dlp/profiles/custom/{profile_id}",
                    account_id=account_id,
                    profile_id=profile_id,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[Profile]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[Profile]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class AsyncCustomResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCustomResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCustomResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCustomResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncCustomResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        name: str,
        ai_context_enabled: bool | Omit = omit,
        allowed_match_count: int | Omit = omit,
        confidence_threshold: Optional[str] | Omit = omit,
        context_awareness: ContextAwarenessParam | Omit = omit,
        data_classes: SequenceNotStr[str] | Omit = omit,
        data_tags: SequenceNotStr[str] | Omit = omit,
        description: Optional[str] | Omit = omit,
        entries: Iterable[custom_create_params.Entry] | Omit = omit,
        ocr_enabled: bool | Omit = omit,
        sensitivity_levels: Iterable[custom_create_params.SensitivityLevel] | Omit = omit,
        shared_entries: Iterable[custom_create_params.SharedEntry] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Profile]:
        """
        Creates a DLP custom profile.

        Args:
          allowed_match_count: Related DLP policies will trigger when the match count exceeds the number set.

          context_awareness: Scan the context of predefined entries to only return matches surrounded by
              keywords.

          data_classes: Data class IDs to associate with the profile.

          data_tags: Data tag IDs to associate with the profile.

          description: The description of the profile.

          sensitivity_levels: Sensitivity levels to associate with the profile.

          shared_entries: Entries from other profiles (e.g. pre-defined Cloudflare profiles, or your
              Microsoft Information Protection profiles).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return cast(
            Optional[Profile],
            await self._post(
                path_template("/accounts/{account_id}/dlp/profiles/custom", account_id=account_id),
                body=await async_maybe_transform(
                    {
                        "name": name,
                        "ai_context_enabled": ai_context_enabled,
                        "allowed_match_count": allowed_match_count,
                        "confidence_threshold": confidence_threshold,
                        "context_awareness": context_awareness,
                        "data_classes": data_classes,
                        "data_tags": data_tags,
                        "description": description,
                        "entries": entries,
                        "ocr_enabled": ocr_enabled,
                        "sensitivity_levels": sensitivity_levels,
                        "shared_entries": shared_entries,
                    },
                    custom_create_params.CustomCreateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[Profile]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[Profile]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def update(
        self,
        profile_id: str,
        *,
        account_id: str,
        name: str,
        ai_context_enabled: bool | Omit = omit,
        allowed_match_count: Optional[int] | Omit = omit,
        confidence_threshold: Optional[str] | Omit = omit,
        context_awareness: ContextAwarenessParam | Omit = omit,
        data_classes: Optional[SequenceNotStr[str]] | Omit = omit,
        data_tags: Optional[SequenceNotStr[str]] | Omit = omit,
        description: Optional[str] | Omit = omit,
        entries: Optional[Iterable[custom_update_params.Entry]] | Omit = omit,
        ocr_enabled: bool | Omit = omit,
        sensitivity_levels: Optional[Iterable[custom_update_params.SensitivityLevel]] | Omit = omit,
        shared_entries: Iterable[custom_update_params.SharedEntry] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Profile]:
        """
        Updates a DLP custom profile.

        Args:
          context_awareness: Scan the context of predefined entries to only return matches surrounded by
              keywords.

          data_classes: Data class IDs to associate with the profile. If omitted, existing associations
              are unchanged.

          data_tags: Data tag IDs to associate with the profile. If omitted, existing associations
              are unchanged.

          description: The description of the profile.

          entries: Custom entries from this profile. If this field is omitted, entries owned by
              this profile will not be changed.

          sensitivity_levels: Sensitivity levels to associate with the profile. If omitted, existing
              associations are unchanged.

          shared_entries: Other entries, e.g. predefined or integration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not profile_id:
            raise ValueError(f"Expected a non-empty value for `profile_id` but received {profile_id!r}")
        return cast(
            Optional[Profile],
            await self._put(
                path_template(
                    "/accounts/{account_id}/dlp/profiles/custom/{profile_id}",
                    account_id=account_id,
                    profile_id=profile_id,
                ),
                body=await async_maybe_transform(
                    {
                        "name": name,
                        "ai_context_enabled": ai_context_enabled,
                        "allowed_match_count": allowed_match_count,
                        "confidence_threshold": confidence_threshold,
                        "context_awareness": context_awareness,
                        "data_classes": data_classes,
                        "data_tags": data_tags,
                        "description": description,
                        "entries": entries,
                        "ocr_enabled": ocr_enabled,
                        "sensitivity_levels": sensitivity_levels,
                        "shared_entries": shared_entries,
                    },
                    custom_update_params.CustomUpdateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[Profile]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[Profile]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def delete(
        self,
        profile_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Deletes a DLP custom profile.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not profile_id:
            raise ValueError(f"Expected a non-empty value for `profile_id` but received {profile_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/dlp/profiles/custom/{profile_id}", account_id=account_id, profile_id=profile_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    async def get(
        self,
        profile_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Profile]:
        """
        Fetches a custom DLP profile by id.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not profile_id:
            raise ValueError(f"Expected a non-empty value for `profile_id` but received {profile_id!r}")
        return cast(
            Optional[Profile],
            await self._get(
                path_template(
                    "/accounts/{account_id}/dlp/profiles/custom/{profile_id}",
                    account_id=account_id,
                    profile_id=profile_id,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[Profile]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[Profile]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class CustomResourceWithRawResponse:
    def __init__(self, custom: CustomResource) -> None:
        self._custom = custom

        self.create = to_raw_response_wrapper(
            custom.create,
        )
        self.update = to_raw_response_wrapper(
            custom.update,
        )
        self.delete = to_raw_response_wrapper(
            custom.delete,
        )
        self.get = to_raw_response_wrapper(
            custom.get,
        )


class AsyncCustomResourceWithRawResponse:
    def __init__(self, custom: AsyncCustomResource) -> None:
        self._custom = custom

        self.create = async_to_raw_response_wrapper(
            custom.create,
        )
        self.update = async_to_raw_response_wrapper(
            custom.update,
        )
        self.delete = async_to_raw_response_wrapper(
            custom.delete,
        )
        self.get = async_to_raw_response_wrapper(
            custom.get,
        )


class CustomResourceWithStreamingResponse:
    def __init__(self, custom: CustomResource) -> None:
        self._custom = custom

        self.create = to_streamed_response_wrapper(
            custom.create,
        )
        self.update = to_streamed_response_wrapper(
            custom.update,
        )
        self.delete = to_streamed_response_wrapper(
            custom.delete,
        )
        self.get = to_streamed_response_wrapper(
            custom.get,
        )


class AsyncCustomResourceWithStreamingResponse:
    def __init__(self, custom: AsyncCustomResource) -> None:
        self._custom = custom

        self.create = async_to_streamed_response_wrapper(
            custom.create,
        )
        self.update = async_to_streamed_response_wrapper(
            custom.update,
        )
        self.delete = async_to_streamed_response_wrapper(
            custom.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            custom.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["CustomTrustStoreCreateParams"]


class CustomTrustStoreCreateParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    certificate: Required[str]
    """The root CA certificate in PEM format.

    Only root CA certificates are accepted; intermediate and leaf certificates are
    not supported.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["DeploymentDeleteParams"]


class DeploymentDeleteParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    project_name: Required[str]
    """Name of the project."""

    force: bool
    """
    Allow deletion of aliased non-production deployments when a normal delete would
    be rejected.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

__all__ = ["ItemDeleteParams", "Item"]


class ItemDeleteParams(TypedDict, total=False):
    account_id: Required[str]
    """The Account ID for this resource."""

    items: Iterable[Item]


class Item(TypedDict, total=False):
    id: Required[str]
    """Defines the unique ID of the item in the List."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Iterable, Optional, cast
from typing_extensions import Literal, overload

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import path_template, required_args, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ....types.d1 import (
    database_raw_params,
    database_edit_params,
    database_list_params,
    database_query_params,
    database_create_params,
    database_export_params,
    database_import_params,
    database_update_params,
)
from .time_travel import (
    TimeTravelResource,
    AsyncTimeTravelResource,
    TimeTravelResourceWithRawResponse,
    AsyncTimeTravelResourceWithRawResponse,
    TimeTravelResourceWithStreamingResponse,
    AsyncTimeTravelResourceWithStreamingResponse,
)
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncSinglePage, AsyncSinglePage, SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ....types.d1.d1 import D1
from ...._base_client import AsyncPaginator, make_request_options
from ....types.d1.query_result import QueryResult
from ....types.d1.database_raw_response import DatabaseRawResponse
from ....types.d1.database_list_response import DatabaseListResponse
from ....types.d1.database_export_response import DatabaseExportResponse
from ....types.d1.database_import_response import DatabaseImportResponse

__all__ = ["DatabaseResource", "AsyncDatabaseResource"]


class DatabaseResource(SyncAPIResource):
    @cached_property
    def time_travel(self) -> TimeTravelResource:
        return TimeTravelResource(self._client)

    @cached_property
    def with_raw_response(self) -> DatabaseResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return DatabaseResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DatabaseResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return DatabaseResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        name: str,
        jurisdiction: Literal["eu", "fedramp"] | Omit = omit,
        primary_location_hint: Literal["wnam", "enam", "weur", "eeur", "apac", "oc"] | Omit = omit,
        read_replication: database_create_params.ReadReplication | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> D1:
        """
        Returns the created D1 database.

        Args:
          account_id: Account identifier tag.

          name: D1 database name.

          jurisdiction: Specify the location to restrict the D1 database to run and store data. If this
              option is present, the location hint is ignored.

          primary_location_hint: Specify the region to create the D1 primary, if available. If this option is
              omitted, the D1 will be created as close as possible to the current user.

          read_replication: Configuration for D1 read replication.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/d1/database", account_id=account_id),
            body=maybe_transform(
                {
                    "name": name,
                    "jurisdiction": jurisdiction,
                    "primary_location_hint": primary_location_hint,
                    "read_replication": read_replication,
                },
                database_create_params.DatabaseCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[D1]._unwrapper,
            ),
            cast_to=cast(Type[D1], ResultWrapper[D1]),
        )

    def update(
        self,
        database_id: str,
        *,
        account_id: str,
        read_replication: database_update_params.ReadReplication,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> D1:
        """
        Updates the specified D1 database.

        Args:
          account_id: Account identifier tag.

          database_id: D1 database identifier (UUID).

          read_replication: Configuration for D1 read replication.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not database_id:
            raise ValueError(f"Expected a non-empty value for `database_id` but received {database_id!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/d1/database/{database_id}", account_id=account_id, database_id=database_id
            ),
            body=maybe_transform({"read_replication": read_replication}, database_update_params.DatabaseUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[D1]._unwrapper,
            ),
            cast_to=cast(Type[D1], ResultWrapper[D1]),
        )

    def list(
        self,
        *,
        account_id: str,
        name: str | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[DatabaseListResponse]:
        """
        Returns a list of D1 databases.

        Args:
          account_id: Account identifier tag.

          name: a database name to search for.

          page: Page number of paginated results.

          per_page: Number of items per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/d1/database", account_id=account_id),
            page=SyncV4PagePaginationArray[DatabaseListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "name": name,
                        "page": page,
                        "per_page": per_page,
                    },
                    database_list_params.DatabaseListParams,
                ),
            ),
            model=DatabaseListResponse,
        )

    def delete(
        self,
        database_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Deletes the specified D1 database.

        Args:
          account_id: Account identifier tag.

          database_id: D1 database identifier (UUID).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not database_id:
            raise ValueError(f"Expected a non-empty value for `database_id` but received {database_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/d1/database/{database_id}", account_id=account_id, database_id=database_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    def edit(
        self,
        database_id: str,
        *,
        account_id: str,
        read_replication: database_edit_params.ReadReplication | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> D1:
        """
        Updates partially the specified D1 database.

        Args:
          account_id: Account identifier tag.

          database_id: D1 database identifier (UUID).

          read_replication: Configuration for D1 read replication.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not database_id:
            raise ValueError(f"Expected a non-empty value for `database_id` but received {database_id!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/d1/database/{database_id}", account_id=account_id, database_id=database_id
            ),
            body=maybe_transform({"read_replication": read_replication}, database_edit_params.DatabaseEditParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[D1]._unwrapper,
            ),
            cast_to=cast(Type[D1], ResultWrapper[D1]),
        )

    def export(
        self,
        database_id: str,
        *,
        account_id: str,
        output_format: Literal["polling"],
        current_bookmark: str | Omit = omit,
        dump_options: database_export_params.DumpOptions | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatabaseExportResponse:
        """Returns a URL where the SQL contents of your D1 can be downloaded.

        Note: this
        process may take some time for larger DBs, during which your D1 will be
        unavailable to serve queries. To avoid blocking your DB unnecessarily, an
        in-progress export must be continually polled or will automatically cancel.

        Args:
          account_id: Account identifier tag.

          database_id: D1 database identifier (UUID).

          output_format: Specifies that you will poll this endpoint until the export completes

          current_bookmark: To poll an in-progress export, provide the current bookmark (returned by your
              first polling response)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not database_id:
            raise ValueError(f"Expected a non-empty value for `database_id` but received {database_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/d1/database/{database_id}/export",
                account_id=account_id,
                database_id=database_id,
            ),
            body=maybe_transform(
                {
                    "output_format": output_format,
                    "current_bookmark": current_bookmark,
                    "dump_options": dump_options,
                },
                database_export_params.DatabaseExportParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[DatabaseExportResponse]._unwrapper,
            ),
            cast_to=cast(Type[DatabaseExportResponse], ResultWrapper[DatabaseExportResponse]),
        )

    def get(
        self,
        database_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> D1:
        """
        Returns the specified D1 database.

        Args:
          account_id: Account identifier tag.

          database_id: D1 database identifier (UUID).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not database_id:
            raise ValueError(f"Expected a non-empty value for `database_id` but received {database_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/d1/database/{database_id}", account_id=account_id, database_id=database_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[D1]._unwrapper,
            ),
            cast_to=cast(Type[D1], ResultWrapper[D1]),
        )

    @overload
    def import_(
        self,
        database_id: str,
        *,
        account_id: str,
        action: Literal["init"],
        etag: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatabaseImportResponse:
        """
        Generates a temporary URL for uploading an SQL file to, then instructing the D1
        to import it and polling it for status updates. Imports block the D1 for their
        duration.

        Args:
          account_id: Account identifier tag.

          database_id: D1 database identifier (UUID).

          action: Indicates you have a new SQL file to upload.

          etag: Required when action is 'init' or 'ingest'. An md5 hash of the file you're
              uploading. Used to check if it already exists, and validate its contents before
              ingesting.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def import_(
        self,
        database_id: str,
        *,
        account_id: str,
        action: Literal["ingest"],
        etag: str,
        filename: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatabaseImportResponse:
        """
        Generates a temporary URL for uploading an SQL file to, then instructing the D1
        to import it and polling it for status updates. Imports block the D1 for their
        duration.

        Args:
          account_id: Account identifier tag.

          database_id: D1 database identifier (UUID).

          action: Indicates you've finished uploading to tell the D1 to start consuming it

          etag: An md5 hash of the file you're uploading. Used to check if it already exists,
              and validate its contents before ingesting.

          filename: The filename you have successfully uploaded.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def import_(
        self,
        database_id: str,
        *,
        account_id: str,
        action: Literal["poll"],
        current_bookmark: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatabaseImportResponse:
        """
        Generates a temporary URL for uploading an SQL file to, then instructing the D1
        to import it and polling it for status updates. Imports block the D1 for their
        duration.

        Args:
          account_id: Account identifier tag.

          database_id: D1 database identifier (UUID).

          action: Indicates you've finished uploading to tell the D1 to start consuming it

          current_bookmark: This identifies the currently-running import, checking its status.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(
        ["account_id", "action", "etag"],
        ["account_id", "action", "etag", "filename"],
        ["account_id", "action", "current_bookmark"],
    )
    def import_(
        self,
        database_id: str,
        *,
        account_id: str,
        action: Literal["init"] | Literal["ingest"] | Literal["poll"],
        etag: str | Omit = omit,
        filename: str | Omit = omit,
        current_bookmark: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatabaseImportResponse:
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not database_id:
            raise ValueError(f"Expected a non-empty value for `database_id` but received {database_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/d1/database/{database_id}/import",
                account_id=account_id,
                database_id=database_id,
            ),
            body=maybe_transform(
                {
                    "action": action,
                    "etag": etag,
                    "filename": filename,
                    "current_bookmark": current_bookmark,
                },
                database_import_params.DatabaseImportParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[DatabaseImportResponse]._unwrapper,
            ),
            cast_to=cast(Type[DatabaseImportResponse], ResultWrapper[DatabaseImportResponse]),
        )

    @overload
    def query(
        self,
        database_id: str,
        *,
        account_id: str,
        sql: str,
        params: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[QueryResult]:
        """
        Returns the query result as an object.

        Args:
          account_id: Account identifier tag.

          database_id: D1 database identifier (UUID).

          sql: Your SQL query. Supports multiple statements, joined by semicolons, which will
              be executed as a batch.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def query(
        self,
        database_id: str,
        *,
        account_id: str,
        batch: Iterable[database_query_params.MultipleQueriesBatch],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[QueryResult]:
        """
        Returns the query result as an object.

        Args:
          account_id: Account identifier tag.

          database_id: D1 database identifier (UUID).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["account_id", "sql"], ["account_id", "batch"])
    def query(
        self,
        database_id: str,
        *,
        account_id: str,
        sql: str | Omit = omit,
        params: SequenceNotStr[str] | Omit = omit,
        batch: Iterable[database_query_params.MultipleQueriesBatch] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[QueryResult]:
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not database_id:
            raise ValueError(f"Expected a non-empty value for `database_id` but received {database_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/d1/database/{database_id}/query", account_id=account_id, database_id=database_id
            ),
            page=SyncSinglePage[QueryResult],
            body=maybe_transform(
                {
                    "sql": sql,
                    "params": params,
                    "batch": batch,
                },
                database_query_params.DatabaseQueryParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=QueryResult,
            method="post",
        )

    @overload
    def raw(
        self,
        database_id: str,
        *,
        account_id: str,
        sql: str,
        params: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[DatabaseRawResponse]:
        """Returns the query result rows as arrays rather than objects.

        This is a
        performance-optimized version of the /query endpoint.

        Args:
          account_id: Account identifier tag.

          database_id: D1 database identifier (UUID).

          sql: Your SQL query. Supports multiple statements, joined by semicolons, which will
              be executed as a batch.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def raw(
        self,
        database_id: str,
        *,
        account_id: str,
        batch: Iterable[database_raw_params.MultipleQueriesBatch],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[DatabaseRawResponse]:
        """Returns the query result rows as arrays rather than objects.

        This is a
        performance-optimized version of the /query endpoint.

        Args:
          account_id: Account identifier tag.

          database_id: D1 database identifier (UUID).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["account_id", "sql"], ["account_id", "batch"])
    def raw(
        self,
        database_id: str,
        *,
        account_id: str,
        sql: str | Omit = omit,
        params: SequenceNotStr[str] | Omit = omit,
        batch: Iterable[database_raw_params.MultipleQueriesBatch] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[DatabaseRawResponse]:
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not database_id:
            raise ValueError(f"Expected a non-empty value for `database_id` but received {database_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/d1/database/{database_id}/raw", account_id=account_id, database_id=database_id
            ),
            page=SyncSinglePage[DatabaseRawResponse],
            body=maybe_transform(
                {
                    "sql": sql,
                    "params": params,
                    "batch": batch,
                },
                database_raw_params.DatabaseRawParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=DatabaseRawResponse,
            method="post",
        )


class AsyncDatabaseResource(AsyncAPIResource):
    @cached_property
    def time_travel(self) -> AsyncTimeTravelResource:
        return AsyncTimeTravelResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncDatabaseResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDatabaseResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDatabaseResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncDatabaseResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        name: str,
        jurisdiction: Literal["eu", "fedramp"] | Omit = omit,
        primary_location_hint: Literal["wnam", "enam", "weur", "eeur", "apac", "oc"] | Omit = omit,
        read_replication: database_create_params.ReadReplication | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> D1:
        """
        Returns the created D1 database.

        Args:
          account_id: Account identifier tag.

          name: D1 database name.

          jurisdiction: Specify the location to restrict the D1 database to run and store data. If this
              option is present, the location hint is ignored.

          primary_location_hint: Specify the region to create the D1 primary, if available. If this option is
              omitted, the D1 will be created as close as possible to the current user.

          read_replication: Configuration for D1 read replication.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/d1/database", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "name": name,
                    "jurisdiction": jurisdiction,
                    "primary_location_hint": primary_location_hint,
                    "read_replication": read_replication,
                },
                database_create_params.DatabaseCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[D1]._unwrapper,
            ),
            cast_to=cast(Type[D1], ResultWrapper[D1]),
        )

    async def update(
        self,
        database_id: str,
        *,
        account_id: str,
        read_replication: database_update_params.ReadReplication,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> D1:
        """
        Updates the specified D1 database.

        Args:
          account_id: Account identifier tag.

          database_id: D1 database identifier (UUID).

          read_replication: Configuration for D1 read replication.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not database_id:
            raise ValueError(f"Expected a non-empty value for `database_id` but received {database_id!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/d1/database/{database_id}", account_id=account_id, database_id=database_id
            ),
            body=await async_maybe_transform(
                {"read_replication": read_replication}, database_update_params.DatabaseUpdateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[D1]._unwrapper,
            ),
            cast_to=cast(Type[D1], ResultWrapper[D1]),
        )

    def list(
        self,
        *,
        account_id: str,
        name: str | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[DatabaseListResponse, AsyncV4PagePaginationArray[DatabaseListResponse]]:
        """
        Returns a list of D1 databases.

        Args:
          account_id: Account identifier tag.

          name: a database name to search for.

          page: Page number of paginated results.

          per_page: Number of items per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/d1/database", account_id=account_id),
            page=AsyncV4PagePaginationArray[DatabaseListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "name": name,
                        "page": page,
                        "per_page": per_page,
                    },
                    database_list_params.DatabaseListParams,
                ),
            ),
            model=DatabaseListResponse,
        )

    async def delete(
        self,
        database_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Deletes the specified D1 database.

        Args:
          account_id: Account identifier tag.

          database_id: D1 database identifier (UUID).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not database_id:
            raise ValueError(f"Expected a non-empty value for `database_id` but received {database_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/d1/database/{database_id}", account_id=account_id, database_id=database_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    async def edit(
        self,
        database_id: str,
        *,
        account_id: str,
        read_replication: database_edit_params.ReadReplication | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> D1:
        """
        Updates partially the specified D1 database.

        Args:
          account_id: Account identifier tag.

          database_id: D1 database identifier (UUID).

          read_replication: Configuration for D1 read replication.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not database_id:
            raise ValueError(f"Expected a non-empty value for `database_id` but received {database_id!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/d1/database/{database_id}", account_id=account_id, database_id=database_id
            ),
            body=await async_maybe_transform(
                {"read_replication": read_replication}, database_edit_params.DatabaseEditParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[D1]._unwrapper,
            ),
            cast_to=cast(Type[D1], ResultWrapper[D1]),
        )

    async def export(
        self,
        database_id: str,
        *,
        account_id: str,
        output_format: Literal["polling"],
        current_bookmark: str | Omit = omit,
        dump_options: database_export_params.DumpOptions | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatabaseExportResponse:
        """Returns a URL where the SQL contents of your D1 can be downloaded.

        Note: this
        process may take some time for larger DBs, during which your D1 will be
        unavailable to serve queries. To avoid blocking your DB unnecessarily, an
        in-progress export must be continually polled or will automatically cancel.

        Args:
          account_id: Account identifier tag.

          database_id: D1 database identifier (UUID).

          output_format: Specifies that you will poll this endpoint until the export completes

          current_bookmark: To poll an in-progress export, provide the current bookmark (returned by your
              first polling response)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not database_id:
            raise ValueError(f"Expected a non-empty value for `database_id` but received {database_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/d1/database/{database_id}/export",
                account_id=account_id,
                database_id=database_id,
            ),
            body=await async_maybe_transform(
                {
                    "output_format": output_format,
                    "current_bookmark": current_bookmark,
                    "dump_options": dump_options,
                },
                database_export_params.DatabaseExportParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[DatabaseExportResponse]._unwrapper,
            ),
            cast_to=cast(Type[DatabaseExportResponse], ResultWrapper[DatabaseExportResponse]),
        )

    async def get(
        self,
        database_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> D1:
        """
        Returns the specified D1 database.

        Args:
          account_id: Account identifier tag.

          database_id: D1 database identifier (UUID).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not database_id:
            raise ValueError(f"Expected a non-empty value for `database_id` but received {database_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/d1/database/{database_id}", account_id=account_id, database_id=database_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[D1]._unwrapper,
            ),
            cast_to=cast(Type[D1], ResultWrapper[D1]),
        )

    @overload
    async def import_(
        self,
        database_id: str,
        *,
        account_id: str,
        action: Literal["init"],
        etag: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatabaseImportResponse:
        """
        Generates a temporary URL for uploading an SQL file to, then instructing the D1
        to import it and polling it for status updates. Imports block the D1 for their
        duration.

        Args:
          account_id: Account identifier tag.

          database_id: D1 database identifier (UUID).

          action: Indicates you have a new SQL file to upload.

          etag: Required when action is 'init' or 'ingest'. An md5 hash of the file you're
              uploading. Used to check if it already exists, and validate its contents before
              ingesting.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def import_(
        self,
        database_id: str,
        *,
        account_id: str,
        action: Literal["ingest"],
        etag: str,
        filename: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatabaseImportResponse:
        """
        Generates a temporary URL for uploading an SQL file to, then instructing the D1
        to import it and polling it for status updates. Imports block the D1 for their
        duration.

        Args:
          account_id: Account identifier tag.

          database_id: D1 database identifier (UUID).

          action: Indicates you've finished uploading to tell the D1 to start consuming it

          etag: An md5 hash of the file you're uploading. Used to check if it already exists,
              and validate its contents before ingesting.

          filename: The filename you have successfully uploaded.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def import_(
        self,
        database_id: str,
        *,
        account_id: str,
        action: Literal["poll"],
        current_bookmark: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatabaseImportResponse:
        """
        Generates a temporary URL for uploading an SQL file to, then instructing the D1
        to import it and polling it for status updates. Imports block the D1 for their
        duration.

        Args:
          account_id: Account identifier tag.

          database_id: D1 database identifier (UUID).

          action: Indicates you've finished uploading to tell the D1 to start consuming it

          current_bookmark: This identifies the currently-running import, checking its status.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(
        ["account_id", "action", "etag"],
        ["account_id", "action", "etag", "filename"],
        ["account_id", "action", "current_bookmark"],
    )
    async def import_(
        self,
        database_id: str,
        *,
        account_id: str,
        action: Literal["init"] | Literal["ingest"] | Literal["poll"],
        etag: str | Omit = omit,
        filename: str | Omit = omit,
        current_bookmark: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatabaseImportResponse:
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not database_id:
            raise ValueError(f"Expected a non-empty value for `database_id` but received {database_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/d1/database/{database_id}/import",
                account_id=account_id,
                database_id=database_id,
            ),
            body=await async_maybe_transform(
                {
                    "action": action,
                    "etag": etag,
                    "filename": filename,
                    "current_bookmark": current_bookmark,
                },
                database_import_params.DatabaseImportParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[DatabaseImportResponse]._unwrapper,
            ),
            cast_to=cast(Type[DatabaseImportResponse], ResultWrapper[DatabaseImportResponse]),
        )

    @overload
    def query(
        self,
        database_id: str,
        *,
        account_id: str,
        sql: str,
        params: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[QueryResult, AsyncSinglePage[QueryResult]]:
        """
        Returns the query result as an object.

        Args:
          account_id: Account identifier tag.

          database_id: D1 database identifier (UUID).

          sql: Your SQL query. Supports multiple statements, joined by semicolons, which will
              be executed as a batch.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def query(
        self,
        database_id: str,
        *,
        account_id: str,
        batch: Iterable[database_query_params.MultipleQueriesBatch],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[QueryResult, AsyncSinglePage[QueryResult]]:
        """
        Returns the query result as an object.

        Args:
          account_id: Account identifier tag.

          database_id: D1 database identifier (UUID).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["account_id", "sql"], ["account_id", "batch"])
    def query(
        self,
        database_id: str,
        *,
        account_id: str,
        sql: str | Omit = omit,
        params: SequenceNotStr[str] | Omit = omit,
        batch: Iterable[database_query_params.MultipleQueriesBatch] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[QueryResult, AsyncSinglePage[QueryResult]]:
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not database_id:
            raise ValueError(f"Expected a non-empty value for `database_id` but received {database_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/d1/database/{database_id}/query", account_id=account_id, database_id=database_id
            ),
            page=AsyncSinglePage[QueryResult],
            body=maybe_transform(
                {
                    "sql": sql,
                    "params": params,
                    "batch": batch,
                },
                database_query_params.DatabaseQueryParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=QueryResult,
            method="post",
        )

    @overload
    def raw(
        self,
        database_id: str,
        *,
        account_id: str,
        sql: str,
        params: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[DatabaseRawResponse, AsyncSinglePage[DatabaseRawResponse]]:
        """Returns the query result rows as arrays rather than objects.

        This is a
        performance-optimized version of the /query endpoint.

        Args:
          account_id: Account identifier tag.

          database_id: D1 database identifier (UUID).

          sql: Your SQL query. Supports multiple statements, joined by semicolons, which will
              be executed as a batch.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def raw(
        self,
        database_id: str,
        *,
        account_id: str,
        batch: Iterable[database_raw_params.MultipleQueriesBatch],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[DatabaseRawResponse, AsyncSinglePage[DatabaseRawResponse]]:
        """Returns the query result rows as arrays rather than objects.

        This is a
        performance-optimized version of the /query endpoint.

        Args:
          account_id: Account identifier tag.

          database_id: D1 database identifier (UUID).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["account_id", "sql"], ["account_id", "batch"])
    def raw(
        self,
        database_id: str,
        *,
        account_id: str,
        sql: str | Omit = omit,
        params: SequenceNotStr[str] | Omit = omit,
        batch: Iterable[database_raw_params.MultipleQueriesBatch] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[DatabaseRawResponse, AsyncSinglePage[DatabaseRawResponse]]:
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not database_id:
            raise ValueError(f"Expected a non-empty value for `database_id` but received {database_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/d1/database/{database_id}/raw", account_id=account_id, database_id=database_id
            ),
            page=AsyncSinglePage[DatabaseRawResponse],
            body=maybe_transform(
                {
                    "sql": sql,
                    "params": params,
                    "batch": batch,
                },
                database_raw_params.DatabaseRawParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=DatabaseRawResponse,
            method="post",
        )


class DatabaseResourceWithRawResponse:
    def __init__(self, database: DatabaseResource) -> None:
        self._database = database

        self.create = to_raw_response_wrapper(
            database.create,
        )
        self.update = to_raw_response_wrapper(
            database.update,
        )
        self.list = to_raw_response_wrapper(
            database.list,
        )
        self.delete = to_raw_response_wrapper(
            database.delete,
        )
        self.edit = to_raw_response_wrapper(
            database.edit,
        )
        self.export = to_raw_response_wrapper(
            database.export,
        )
        self.get = to_raw_response_wrapper(
            database.get,
        )
        self.import_ = to_raw_response_wrapper(
            database.import_,
        )
        self.query = to_raw_response_wrapper(
            database.query,
        )
        self.raw = to_raw_response_wrapper(
            database.raw,
        )

    @cached_property
    def time_travel(self) -> TimeTravelResourceWithRawResponse:
        return TimeTravelResourceWithRawResponse(self._database.time_travel)


class AsyncDatabaseResourceWithRawResponse:
    def __init__(self, database: AsyncDatabaseResource) -> None:
        self._database = database

        self.create = async_to_raw_response_wrapper(
            database.create,
        )
        self.update = async_to_raw_response_wrapper(
            database.update,
        )
        self.list = async_to_raw_response_wrapper(
            database.list,
        )
        self.delete = async_to_raw_response_wrapper(
            database.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            database.edit,
        )
        self.export = async_to_raw_response_wrapper(
            database.export,
        )
        self.get = async_to_raw_response_wrapper(
            database.get,
        )
        self.import_ = async_to_raw_response_wrapper(
            database.import_,
        )
        self.query = async_to_raw_response_wrapper(
            database.query,  # pyright: ignore[reportArgumentType]
        )
        self.raw = async_to_raw_response_wrapper(
            database.raw,  # pyright: ignore[reportArgumentType]
        )

    @cached_property
    def time_travel(self) -> AsyncTimeTravelResourceWithRawResponse:
        return AsyncTimeTravelResourceWithRawResponse(self._database.time_travel)


class DatabaseResourceWithStreamingResponse:
    def __init__(self, database: DatabaseResource) -> None:
        self._database = database

        self.create = to_streamed_response_wrapper(
            database.create,
        )
        self.update = to_streamed_response_wrapper(
            database.update,
        )
        self.list = to_streamed_response_wrapper(
            database.list,
        )
        self.delete = to_streamed_response_wrapper(
            database.delete,
        )
        self.edit = to_streamed_response_wrapper(
            database.edit,
        )
        self.export = to_streamed_response_wrapper(
            database.export,
        )
        self.get = to_streamed_response_wrapper(
            database.get,
        )
        self.import_ = to_streamed_response_wrapper(
            database.import_,
        )
        self.query = to_streamed_response_wrapper(
            database.query,
        )
        self.raw = to_streamed_response_wrapper(
            database.raw,
        )

    @cached_property
    def time_travel(self) -> TimeTravelResourceWithStreamingResponse:
        return TimeTravelResourceWithStreamingResponse(self._database.time_travel)


class AsyncDatabaseResourceWithStreamingResponse:
    def __init__(self, database: AsyncDatabaseResource) -> None:
        self._database = database

        self.create = async_to_streamed_response_wrapper(
            database.create,
        )
        self.update = async_to_streamed_response_wrapper(
            database.update,
        )
        self.list = async_to_streamed_response_wrapper(
            database.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            database.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            database.edit,
        )
        self.export = async_to_streamed_response_wrapper(
            database.export,
        )
        self.get = async_to_streamed_response_wrapper(
            database.get,
        )
        self.import_ = async_to_streamed_response_wrapper(
            database.import_,
        )
        self.query = async_to_streamed_response_wrapper(
            database.query,  # pyright: ignore[reportArgumentType]
        )
        self.raw = async_to_streamed_response_wrapper(
            database.raw,  # pyright: ignore[reportArgumentType]
        )

    @cached_property
    def time_travel(self) -> AsyncTimeTravelResourceWithStreamingResponse:
        return AsyncTimeTravelResourceWithStreamingResponse(self._database.time_travel)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .acls import (
    ACLsResource,
    AsyncACLsResource,
    ACLsResourceWithRawResponse,
    AsyncACLsResourceWithRawResponse,
    ACLsResourceWithStreamingResponse,
    AsyncACLsResourceWithStreamingResponse,
)
from .lans import (
    LANsResource,
    AsyncLANsResource,
    LANsResourceWithRawResponse,
    AsyncLANsResourceWithRawResponse,
    LANsResourceWithStreamingResponse,
    AsyncLANsResourceWithStreamingResponse,
)
from .wans import (
    WANsResource,
    AsyncWANsResource,
    WANsResourceWithRawResponse,
    AsyncWANsResourceWithRawResponse,
    WANsResourceWithStreamingResponse,
    AsyncWANsResourceWithStreamingResponse,
)
from .sites import (
    SitesResource,
    AsyncSitesResource,
    SitesResourceWithRawResponse,
    AsyncSitesResourceWithRawResponse,
    SitesResourceWithStreamingResponse,
    AsyncSitesResourceWithStreamingResponse,
)

__all__ = [
    "ACLsResource",
    "AsyncACLsResource",
    "ACLsResourceWithRawResponse",
    "AsyncACLsResourceWithRawResponse",
    "ACLsResourceWithStreamingResponse",
    "AsyncACLsResourceWithStreamingResponse",
    "LANsResource",
    "AsyncLANsResource",
    "LANsResourceWithRawResponse",
    "AsyncLANsResourceWithRawResponse",
    "LANsResourceWithStreamingResponse",
    "AsyncLANsResourceWithStreamingResponse",
    "WANsResource",
    "AsyncWANsResource",
    "WANsResourceWithRawResponse",
    "AsyncWANsResourceWithRawResponse",
    "WANsResourceWithStreamingResponse",
    "AsyncWANsResourceWithStreamingResponse",
    "SitesResource",
    "AsyncSitesResource",
    "SitesResourceWithRawResponse",
    "AsyncSitesResourceWithRawResponse",
    "SitesResourceWithStreamingResponse",
    "AsyncSitesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

from ..._types import SequenceNotStr

__all__ = ["RatePlan"]


class RatePlan(TypedDict, total=False):
    """The rate plan applied to the subscription."""

    id: Literal[
        "free",
        "lite",
        "pro",
        "pro_plus",
        "business",
        "enterprise",
        "partners_free",
        "partners_pro",
        "partners_business",
        "partners_enterprise",
    ]
    """The ID of the rate plan."""

    currency: str
    """The currency applied to the rate plan subscription."""

    externally_managed: bool
    """Whether this rate plan is managed externally from Cloudflare."""

    is_contract: bool
    """Whether a rate plan is enterprise-based (or newly adopted term contract)."""

    public_name: str
    """The full name of the rate plan."""

    scope: str
    """The scope that this rate plan applies to."""

    sets: SequenceNotStr[str]
    """The list of sets this rate plan applies to. Returns array of strings."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from .type import Type

__all__ = ["ZoneCreateParams", "Account"]


class ZoneCreateParams(TypedDict, total=False):
    account: Required[Account]

    name: Required[str]
    """The domain name.

    Per [RFC 1035](https://datatracker.ietf.org/doc/html/rfc1035#section-2.3.4) the
    overall zone name can be up to 253 characters, with each segment ("label") not
    exceeding 63 characters.
    """

    type: Type
    """A full zone implies that DNS is hosted with Cloudflare.

    A partial zone is typically a partner-hosted zone or a CNAME setup.
    """


class Account(TypedDict, total=False):
    id: str
    """Identifier"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from typing_extensions import TypeAlias

from ..hostname import Hostname
from ..redirect import Redirect
from ...._models import BaseModel

__all__ = [
    "ItemGetResponse",
    "ListsListItemIPFull",
    "ListsListItemHostnameFull",
    "ListsListItemRedirectFull",
    "ListsListItemASNFull",
]


class ListsListItemIPFull(BaseModel):
    id: str
    """Defines the unique ID of the item in the List."""

    created_on: str
    """The RFC 3339 timestamp of when the list was created."""

    ip: str
    """An IPv4 address, an IPv4 CIDR, an IPv6 address, or an IPv6 CIDR."""

    modified_on: str
    """The RFC 3339 timestamp of when the list was last modified."""

    comment: Optional[str] = None
    """Defines an informative summary of the list item."""


class ListsListItemHostnameFull(BaseModel):
    id: str
    """Defines the unique ID of the item in the List."""

    created_on: str
    """The RFC 3339 timestamp of when the list was created."""

    hostname: Hostname
    """
    Valid characters for hostnames are ASCII(7) letters from a to z, the digits from
    0 to 9, wildcards (\\**), and the hyphen (-).
    """

    modified_on: str
    """The RFC 3339 timestamp of when the list was last modified."""

    comment: Optional[str] = None
    """Defines an informative summary of the list item."""


class ListsListItemRedirectFull(BaseModel):
    id: str
    """Defines the unique ID of the item in the List."""

    created_on: str
    """The RFC 3339 timestamp of when the list was created."""

    modified_on: str
    """The RFC 3339 timestamp of when the list was last modified."""

    redirect: Redirect
    """The definition of the redirect."""

    comment: Optional[str] = None
    """Defines an informative summary of the list item."""


class ListsListItemASNFull(BaseModel):
    id: str
    """Defines the unique ID of the item in the List."""

    asn: int
    """Defines a non-negative 32 bit integer."""

    created_on: str
    """The RFC 3339 timestamp of when the list was created."""

    modified_on: str
    """The RFC 3339 timestamp of when the list was last modified."""

    comment: Optional[str] = None
    """Defines an informative summary of the list item."""


ItemGetResponse: TypeAlias = Union[
    ListsListItemIPFull, ListsListItemHostnameFull, ListsListItemRedirectFull, ListsListItemASNFull
]
